# Criteo API SDK for Python

## Introduction
API Client Libraries can facilitate your use of the Criteo API allowing you to build unique and customized solutions to serve your businesses and clients.
These libraries can reduce the amount of code you need to write in order to start accessing Criteo programmatically. They also can help expedite troubleshooting, should you encounter any issues.

More information: [https://developers.criteo.com/](https://developers.criteo.com/)

This Python package is automatically generated by the [OpenAPI Generator](https://openapi-generator.tech) project:

- Package version: 2024.01.0.240808

## Requirements

Python 2.7 and 3.5+

## Installation & Usage
### pip install

```sh
pip install criteo-api-marketingsolutions-sdk==2024.01.0.240808
```
(you may need to run `pip` with root permission: `sudo pip install criteo-api-marketingsolutions-sdk==2024.01.0.240808`)

Then import the package:
```python
import criteo_api_marketingsolutions_v2024_01
```

### Manual Installation using [Setuptools](http://pypi.python.org/pypi/setuptools)

Download the code or clone the repository locally, then execute the following command:

```sh
python setup.py install --user
```
(or `sudo python setup.py install` to install the package for all users)

Then import the package:
```python
import criteo_api_marketingsolutions_v2024_01
```

## Example
There are multiple examples for the different OAuth flows that the SDK supports.
- See [test/example_application_with_client_credentials.py](test/example_application_with_client_credentials.py) for an example with Client Credentials.
- See [test/example_application_with_auth_code.py](test/example_application_with_auth_code.py) for an example with Authorization Code.
Once you follow the authorization code flow, you will have a refresh token that has to be used to regenerate access token for future usage. 
    - See [test/example_application_with_refresh_token.py](test/example_application_with_refresh_token.py) for an example with Refresh Token .

## Documentation for API Endpoints

The developers documentation is available at: *https://developers.criteo.com*.

All URIs are relative to *https://api.criteo.com*.

Class | Method | HTTP request | Description
------------ | ------------- | ------------- | -------------


## Documentation For Models

 - [Ad](docs/Ad.md)
 - [AdListResponse](docs/AdListResponse.md)
 - [AdResource](docs/AdResource.md)
 - [AdResponse](docs/AdResponse.md)
 - [AdSetAudienceLinkEntityV1](docs/AdSetAudienceLinkEntityV1.md)
 - [AdSetAudienceLinkEntityV1Resource](docs/AdSetAudienceLinkEntityV1Resource.md)
 - [AdSetAudienceLinkEntityV1Response](docs/AdSetAudienceLinkEntityV1Response.md)
 - [AdSetAudienceLinkInputEntityV1](docs/AdSetAudienceLinkInputEntityV1.md)
 - [AdSetCategoryBid](docs/AdSetCategoryBid.md)
 - [AdSetCategoryBidListResponse](docs/AdSetCategoryBidListResponse.md)
 - [AdSetCategoryBidResource](docs/AdSetCategoryBidResource.md)
 - [AdSetDeliveryLimitations](docs/AdSetDeliveryLimitations.md)
 - [AdSetDeliveryLimitationsV23Q1](docs/AdSetDeliveryLimitationsV23Q1.md)
 - [AdSetDisplayMultiplier](docs/AdSetDisplayMultiplier.md)
 - [AdSetDisplayMultiplierListResponse](docs/AdSetDisplayMultiplierListResponse.md)
 - [AdSetDisplayMultiplierResource](docs/AdSetDisplayMultiplierResource.md)
 - [AdSetFrequencyCapping](docs/AdSetFrequencyCapping.md)
 - [AdSetFrequencyCappingV23Q1](docs/AdSetFrequencyCappingV23Q1.md)
 - [AdSetGeoLocation](docs/AdSetGeoLocation.md)
 - [AdSetGeoLocationV23Q1](docs/AdSetGeoLocationV23Q1.md)
 - [AdSetSearchFilterV23Q1](docs/AdSetSearchFilterV23Q1.md)
 - [AdSetSearchRequestV23Q1](docs/AdSetSearchRequestV23Q1.md)
 - [AdSetTargeting](docs/AdSetTargeting.md)
 - [AdSetTargetingRule](docs/AdSetTargetingRule.md)
 - [AdSetTargetingRuleV23Q1](docs/AdSetTargetingRuleV23Q1.md)
 - [AdSetTargetingV23Q1](docs/AdSetTargetingV23Q1.md)
 - [AdWrite](docs/AdWrite.md)
 - [AdWriteRequest](docs/AdWriteRequest.md)
 - [AdWriteResource](docs/AdWriteResource.md)
 - [AdaptiveAttributes](docs/AdaptiveAttributes.md)
 - [AdaptiveColors](docs/AdaptiveColors.md)
 - [AdaptiveWriteAttributes](docs/AdaptiveWriteAttributes.md)
 - [AlgebraNodeV1](docs/AlgebraNodeV1.md)
 - [ApplicationSummaryModel](docs/ApplicationSummaryModel.md)
 - [ApplicationSummaryModelResource](docs/ApplicationSummaryModelResource.md)
 - [ApplicationSummaryModelResponse](docs/ApplicationSummaryModelResponse.md)
 - [AudienceBulkCreateInputV1](docs/AudienceBulkCreateInputV1.md)
 - [AudienceBulkDeleteInputV1](docs/AudienceBulkDeleteInputV1.md)
 - [AudienceBulkUpdateInputV1](docs/AudienceBulkUpdateInputV1.md)
 - [AudienceComputeSizeEntityV1Resource](docs/AudienceComputeSizeEntityV1Resource.md)
 - [AudienceComputeSizesInputV1](docs/AudienceComputeSizesInputV1.md)
 - [AudienceCreateEntityV1](docs/AudienceCreateEntityV1.md)
 - [AudienceCreateEntityV1Resource](docs/AudienceCreateEntityV1Resource.md)
 - [AudienceDeleteEntityV1Resource](docs/AudienceDeleteEntityV1Resource.md)
 - [AudienceEntityV1](docs/AudienceEntityV1.md)
 - [AudienceEntityV1AudienceSearchMetadataV1ListResponse](docs/AudienceEntityV1AudienceSearchMetadataV1ListResponse.md)
 - [AudienceEntityV1ListResponse](docs/AudienceEntityV1ListResponse.md)
 - [AudienceEntityV1Resource](docs/AudienceEntityV1Resource.md)
 - [AudienceError](docs/AudienceError.md)
 - [AudienceEstimateSizeEntityV1](docs/AudienceEstimateSizeEntityV1.md)
 - [AudienceEstimateSizeEntityV1Resource](docs/AudienceEstimateSizeEntityV1Resource.md)
 - [AudienceEstimateSizeInputV1](docs/AudienceEstimateSizeInputV1.md)
 - [AudienceIdEntityV1ListResponse](docs/AudienceIdEntityV1ListResponse.md)
 - [AudienceIdEntityV1Resource](docs/AudienceIdEntityV1Resource.md)
 - [AudienceNameDescription](docs/AudienceNameDescription.md)
 - [AudienceSearchEntityV1](docs/AudienceSearchEntityV1.md)
 - [AudienceSearchEntityV1Resource](docs/AudienceSearchEntityV1Resource.md)
 - [AudienceSearchInputV1](docs/AudienceSearchInputV1.md)
 - [AudienceSearchMetadataV1](docs/AudienceSearchMetadataV1.md)
 - [AudienceSegmentBulkCreateInputV1](docs/AudienceSegmentBulkCreateInputV1.md)
 - [AudienceSegmentBulkDeleteInputV1](docs/AudienceSegmentBulkDeleteInputV1.md)
 - [AudienceSegmentBulkUpdateInputV1](docs/AudienceSegmentBulkUpdateInputV1.md)
 - [AudienceSegmentComputeSizeEntityV1Resource](docs/AudienceSegmentComputeSizeEntityV1Resource.md)
 - [AudienceSegmentComputeSizesInputV1](docs/AudienceSegmentComputeSizesInputV1.md)
 - [AudienceSegmentCreateEntityV1](docs/AudienceSegmentCreateEntityV1.md)
 - [AudienceSegmentCreateEntityV1Resource](docs/AudienceSegmentCreateEntityV1Resource.md)
 - [AudienceSegmentDeleteEntityV1Resource](docs/AudienceSegmentDeleteEntityV1Resource.md)
 - [AudienceSegmentEntityV1](docs/AudienceSegmentEntityV1.md)
 - [AudienceSegmentEntityV1AudienceSegmentSearchMetadataV1ListResponse](docs/AudienceSegmentEntityV1AudienceSegmentSearchMetadataV1ListResponse.md)
 - [AudienceSegmentEntityV1ListResponse](docs/AudienceSegmentEntityV1ListResponse.md)
 - [AudienceSegmentEntityV1Resource](docs/AudienceSegmentEntityV1Resource.md)
 - [AudienceSegmentEstimateSizeInputV1](docs/AudienceSegmentEstimateSizeInputV1.md)
 - [AudienceSegmentIdEntityV1ListResponse](docs/AudienceSegmentIdEntityV1ListResponse.md)
 - [AudienceSegmentIdEntityV1Resource](docs/AudienceSegmentIdEntityV1Resource.md)
 - [AudienceSegmentSearchEntityV1](docs/AudienceSegmentSearchEntityV1.md)
 - [AudienceSegmentSearchEntityV1Resource](docs/AudienceSegmentSearchEntityV1Resource.md)
 - [AudienceSegmentSearchInputV1](docs/AudienceSegmentSearchInputV1.md)
 - [AudienceSegmentSearchMetadataV1](docs/AudienceSegmentSearchMetadataV1.md)
 - [AudienceSegmentSizeEntityV1](docs/AudienceSegmentSizeEntityV1.md)
 - [AudienceSegmentSizeEntityV1ListResponse](docs/AudienceSegmentSizeEntityV1ListResponse.md)
 - [AudienceSegmentSizeEntityV1Resource](docs/AudienceSegmentSizeEntityV1Resource.md)
 - [AudienceSegmentSizeEstimationEntityV1](docs/AudienceSegmentSizeEstimationEntityV1.md)
 - [AudienceSegmentSizeEstimationEntityV1Resource](docs/AudienceSegmentSizeEstimationEntityV1Resource.md)
 - [AudienceSegmentSizeEstimationV1](docs/AudienceSegmentSizeEstimationV1.md)
 - [AudienceSegmentSizeEstimationV1Resource](docs/AudienceSegmentSizeEstimationV1Resource.md)
 - [AudienceSegmentSizeEstimationV1Response](docs/AudienceSegmentSizeEstimationV1Response.md)
 - [AudienceSegmentUpdateEntityV1](docs/AudienceSegmentUpdateEntityV1.md)
 - [AudienceSegmentUpdateEntityV1Resource](docs/AudienceSegmentUpdateEntityV1Resource.md)
 - [AudienceSizeEntityV1](docs/AudienceSizeEntityV1.md)
 - [AudienceSizeEntityV1ListResponse](docs/AudienceSizeEntityV1ListResponse.md)
 - [AudienceSizeEntityV1Resource](docs/AudienceSizeEntityV1Resource.md)
 - [AudienceSizeEstimationV1](docs/AudienceSizeEstimationV1.md)
 - [AudienceSizeEstimationV1Resource](docs/AudienceSizeEstimationV1Resource.md)
 - [AudienceSizeEstimationV1Response](docs/AudienceSizeEstimationV1Response.md)
 - [AudienceUpdateEntityV1](docs/AudienceUpdateEntityV1.md)
 - [AudienceUpdateEntityV1Resource](docs/AudienceUpdateEntityV1Resource.md)
 - [AudienceWarning](docs/AudienceWarning.md)
 - [BasicAudienceDefinition](docs/BasicAudienceDefinition.md)
 - [BehavioralV1](docs/BehavioralV1.md)
 - [CampaignSearchFiltersV23Q1](docs/CampaignSearchFiltersV23Q1.md)
 - [CampaignSearchRequestV23Q1](docs/CampaignSearchRequestV23Q1.md)
 - [CampaignSpendLimitV23Q1](docs/CampaignSpendLimitV23Q1.md)
 - [CampaignV23Q1](docs/CampaignV23Q1.md)
 - [CampaignV23Q1ListResponse](docs/CampaignV23Q1ListResponse.md)
 - [CampaignV23Q1Resource](docs/CampaignV23Q1Resource.md)
 - [CampaignV23Q1Response](docs/CampaignV23Q1Response.md)
 - [CommonProblem](docs/CommonProblem.md)
 - [ContactListStatisticsEntityV1](docs/ContactListStatisticsEntityV1.md)
 - [ContactListStatisticsEntityV1Resource](docs/ContactListStatisticsEntityV1Resource.md)
 - [ContactListStatisticsEntityV1Response](docs/ContactListStatisticsEntityV1Response.md)
 - [ContactListV1](docs/ContactListV1.md)
 - [ContactlistAmendment](docs/ContactlistAmendment.md)
 - [ContactlistAmendmentAttributes](docs/ContactlistAmendmentAttributes.md)
 - [ContactlistAmendmentRequest](docs/ContactlistAmendmentRequest.md)
 - [ContactlistOperation](docs/ContactlistOperation.md)
 - [ContactlistOperationAttributes](docs/ContactlistOperationAttributes.md)
 - [Coupon](docs/Coupon.md)
 - [CouponListResponse](docs/CouponListResponse.md)
 - [CouponResource](docs/CouponResource.md)
 - [CouponResponse](docs/CouponResponse.md)
 - [CouponSupportedSizes](docs/CouponSupportedSizes.md)
 - [CouponSupportedSizesResource](docs/CouponSupportedSizesResource.md)
 - [CouponSupportedSizesResponse](docs/CouponSupportedSizesResponse.md)
 - [CreateAdSet](docs/CreateAdSet.md)
 - [CreateAdSetBidding](docs/CreateAdSetBidding.md)
 - [CreateAdSetBudget](docs/CreateAdSetBudget.md)
 - [CreateAdSetGeoLocation](docs/CreateAdSetGeoLocation.md)
 - [CreateAdSetRequest](docs/CreateAdSetRequest.md)
 - [CreateAdSetResource](docs/CreateAdSetResource.md)
 - [CreateAdSetSchedule](docs/CreateAdSetSchedule.md)
 - [CreateAdSetTargeting](docs/CreateAdSetTargeting.md)
 - [CreateCampaign](docs/CreateCampaign.md)
 - [CreateCampaignRequest](docs/CreateCampaignRequest.md)
 - [CreateCampaignResource](docs/CreateCampaignResource.md)
 - [CreateCampaignSpendLimit](docs/CreateCampaignSpendLimit.md)
 - [CreateCoupon](docs/CreateCoupon.md)
 - [CreateCouponRequest](docs/CreateCouponRequest.md)
 - [CreateCouponResource](docs/CreateCouponResource.md)
 - [CreateImageSlide](docs/CreateImageSlide.md)
 - [Creative](docs/Creative.md)
 - [CreativeListResponse](docs/CreativeListResponse.md)
 - [CreativeResource](docs/CreativeResource.md)
 - [CreativeResponse](docs/CreativeResponse.md)
 - [CreativeWrite](docs/CreativeWrite.md)
 - [CreativeWriteRequest](docs/CreativeWriteRequest.md)
 - [CreativeWriteResource](docs/CreativeWriteResource.md)
 - [CriteoApiError](docs/CriteoApiError.md)
 - [CriteoApiWarning](docs/CriteoApiWarning.md)
 - [DeleteAudienceContactListResponse](docs/DeleteAudienceContactListResponse.md)
 - [DynamicAttributes](docs/DynamicAttributes.md)
 - [DynamicWriteAttributes](docs/DynamicWriteAttributes.md)
 - [EntityOfPortfolioMessage](docs/EntityOfPortfolioMessage.md)
 - [ErrorCodeResponse](docs/ErrorCodeResponse.md)
 - [GetPortfolioResponse](docs/GetPortfolioResponse.md)
 - [HtmlTagAttributes](docs/HtmlTagAttributes.md)
 - [HtmlTagWriteAttributes](docs/HtmlTagWriteAttributes.md)
 - [ImageAttributes](docs/ImageAttributes.md)
 - [ImageSet](docs/ImageSet.md)
 - [ImageSetBase64](docs/ImageSetBase64.md)
 - [ImageShape](docs/ImageShape.md)
 - [ImageSlide](docs/ImageSlide.md)
 - [ImageWriteAttributes](docs/ImageWriteAttributes.md)
 - [InMarketAudienceSegmentBrandEntityV1](docs/InMarketAudienceSegmentBrandEntityV1.md)
 - [InMarketAudienceSegmentBrandEntityV1ListResponse](docs/InMarketAudienceSegmentBrandEntityV1ListResponse.md)
 - [InMarketAudienceSegmentBrandEntityV1Resource](docs/InMarketAudienceSegmentBrandEntityV1Resource.md)
 - [InMarketAudienceSegmentInterestEntityV1](docs/InMarketAudienceSegmentInterestEntityV1.md)
 - [InMarketAudienceSegmentInterestEntityV1ListResponse](docs/InMarketAudienceSegmentInterestEntityV1ListResponse.md)
 - [InMarketAudienceSegmentInterestEntityV1Resource](docs/InMarketAudienceSegmentInterestEntityV1Resource.md)
 - [InMarketCreateV1](docs/InMarketCreateV1.md)
 - [InMarketSizeEstimationV1](docs/InMarketSizeEstimationV1.md)
 - [InMarketUpdateV1](docs/InMarketUpdateV1.md)
 - [InMarketV1](docs/InMarketV1.md)
 - [LocationCreateV1](docs/LocationCreateV1.md)
 - [LocationSizeEstimationV1](docs/LocationSizeEstimationV1.md)
 - [LocationUpdateV1](docs/LocationUpdateV1.md)
 - [LocationV1](docs/LocationV1.md)
 - [LookalikeCreateV1](docs/LookalikeCreateV1.md)
 - [LookalikeUpdateV1](docs/LookalikeUpdateV1.md)
 - [LookalikeV1](docs/LookalikeV1.md)
 - [ModifyAudienceResponse](docs/ModifyAudienceResponse.md)
 - [NillableAdSetTargetingRule](docs/NillableAdSetTargetingRule.md)
 - [NillableAdSetTargetingRuleV23Q1](docs/NillableAdSetTargetingRuleV23Q1.md)
 - [NillableAdSetTargetingRuleV23Q1Value](docs/NillableAdSetTargetingRuleV23Q1Value.md)
 - [NillableAdSetTargetingRuleValue](docs/NillableAdSetTargetingRuleValue.md)
 - [NillableDateTime](docs/NillableDateTime.md)
 - [NillableDecimal](docs/NillableDecimal.md)
 - [NillableGenderV1](docs/NillableGenderV1.md)
 - [NillableInt32](docs/NillableInt32.md)
 - [NillableString](docs/NillableString.md)
 - [Outcome](docs/Outcome.md)
 - [PatchAdSet](docs/PatchAdSet.md)
 - [PatchAdSetBidding](docs/PatchAdSetBidding.md)
 - [PatchAdSetBudget](docs/PatchAdSetBudget.md)
 - [PatchAdSetCategoryBid](docs/PatchAdSetCategoryBid.md)
 - [PatchAdSetCategoryBidListRequest](docs/PatchAdSetCategoryBidListRequest.md)
 - [PatchAdSetCategoryBidResource](docs/PatchAdSetCategoryBidResource.md)
 - [PatchAdSetCategoryBidResultListResponse](docs/PatchAdSetCategoryBidResultListResponse.md)
 - [PatchAdSetCategoryBidResultResource](docs/PatchAdSetCategoryBidResultResource.md)
 - [PatchAdSetDisplayMultiplier](docs/PatchAdSetDisplayMultiplier.md)
 - [PatchAdSetDisplayMultiplierListRequest](docs/PatchAdSetDisplayMultiplierListRequest.md)
 - [PatchAdSetDisplayMultiplierResource](docs/PatchAdSetDisplayMultiplierResource.md)
 - [PatchAdSetDisplayMultiplierResultListResponse](docs/PatchAdSetDisplayMultiplierResultListResponse.md)
 - [PatchAdSetDisplayMultiplierResultResource](docs/PatchAdSetDisplayMultiplierResultResource.md)
 - [PatchAdSetScheduling](docs/PatchAdSetScheduling.md)
 - [PatchCampaign](docs/PatchCampaign.md)
 - [PatchCampaignListRequest](docs/PatchCampaignListRequest.md)
 - [PatchCampaignSpendLimit](docs/PatchCampaignSpendLimit.md)
 - [PatchCampaignWriteResource](docs/PatchCampaignWriteResource.md)
 - [PatchResultCampaignListResponse](docs/PatchResultCampaignListResponse.md)
 - [PatchResultCampaignReadResource](docs/PatchResultCampaignReadResource.md)
 - [PlacementsReportQueryMessage](docs/PlacementsReportQueryMessage.md)
 - [PlacementsReportQueryMessageListRequest](docs/PlacementsReportQueryMessageListRequest.md)
 - [PlacementsReportQueryMessageResource](docs/PlacementsReportQueryMessageResource.md)
 - [PointOfInterestV1](docs/PointOfInterestV1.md)
 - [PortfolioMessage](docs/PortfolioMessage.md)
 - [ProspectingCreateV1](docs/ProspectingCreateV1.md)
 - [ProspectingUpdateV1](docs/ProspectingUpdateV1.md)
 - [ProspectingV1](docs/ProspectingV1.md)
 - [ReadAdSet](docs/ReadAdSet.md)
 - [ReadAdSetBidding](docs/ReadAdSetBidding.md)
 - [ReadAdSetBiddingV23Q1](docs/ReadAdSetBiddingV23Q1.md)
 - [ReadAdSetBudget](docs/ReadAdSetBudget.md)
 - [ReadAdSetBudgetV23Q1](docs/ReadAdSetBudgetV23Q1.md)
 - [ReadAdSetSchedule](docs/ReadAdSetSchedule.md)
 - [ReadAdSetScheduleV23Q1](docs/ReadAdSetScheduleV23Q1.md)
 - [ReadAdSetV23Q1](docs/ReadAdSetV23Q1.md)
 - [ReadModelAdSetId](docs/ReadModelAdSetId.md)
 - [ReadModelReadAdSet](docs/ReadModelReadAdSet.md)
 - [ReadModelReadAdSetV23Q1](docs/ReadModelReadAdSetV23Q1.md)
 - [RequestsAdSetId](docs/RequestsAdSetId.md)
 - [RequestsPatchAdSet](docs/RequestsPatchAdSet.md)
 - [ResponseReadAdSet](docs/ResponseReadAdSet.md)
 - [ResponseReadAdSetV23Q1](docs/ResponseReadAdSetV23Q1.md)
 - [ResponsesAdSetId](docs/ResponsesAdSetId.md)
 - [ResponsesReadAdSetV23Q1](docs/ResponsesReadAdSetV23Q1.md)
 - [RetargetingCreateV1](docs/RetargetingCreateV1.md)
 - [RetargetingUpdateV1](docs/RetargetingUpdateV1.md)
 - [RetargetingV1](docs/RetargetingV1.md)
 - [Size](docs/Size.md)
 - [StatisticsReportQueryMessage](docs/StatisticsReportQueryMessage.md)
 - [Tag](docs/Tag.md)
 - [TransactionsReportQueryMessage](docs/TransactionsReportQueryMessage.md)
 - [TransactionsReportQueryMessageListRequest](docs/TransactionsReportQueryMessageListRequest.md)
 - [TransactionsReportQueryMessageResource](docs/TransactionsReportQueryMessageResource.md)
 - [TransparencyQueryMessage](docs/TransparencyQueryMessage.md)
 - [TransparencyReport](docs/TransparencyReport.md)
 - [TransparencyReportFile](docs/TransparencyReportFile.md)
 - [TransparencyReportListResponse](docs/TransparencyReportListResponse.md)
 - [TransparencyReportResource](docs/TransparencyReportResource.md)
 - [UpdateCoupon](docs/UpdateCoupon.md)
 - [UpdateCouponRequest](docs/UpdateCouponRequest.md)
 - [UpdateCouponResource](docs/UpdateCouponResource.md)
 - [VideoDetail](docs/VideoDetail.md)
 - [WriteModelAdSetId](docs/WriteModelAdSetId.md)
 - [WriteModelPatchAdSet](docs/WriteModelPatchAdSet.md)


## Disclaimer

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.